/**
 * Настройки приложения
 */
var config = module.exports = {

  /**
   * Название приложения ( пока что используется только для логирования )
   *
   * @type {String}
   */
  name                : 'hello_world',

  /**
   * Корневая директория приложения, в ней по умолчанию ищутся директории с моделями, контроллерами, вьюшками,
   * компонентами, а также относительно нее задаются другие пути в конфигурационном файле
   *
   * @type {String}
   */
  base_dir            : __dirname,

  /**
   * Параметры приложения, которые доступны в любом месте приложения через функцияю {@link Application.get_param}
   *
   * @type {Object}
   * @property {String} your_name ваше имя, которое будет выводиться на главной странице
   */
  params : {
    your_name : 'Andrey'
  },

  /**
   * Настройки компонента отвечающего за перенаправление запросов и генерацию запросов
   *
   * @type {Object}
   */
  router : {
    rules     : {
      ''                      : 'site.index'
    }
  },

  /**
   * Компоненты, загружаемые до инициализации ядра приложения
   * log_router - чтобы видеть этапы инициализации ядра
   *
   * @type {Array}
   */
  preload_components : [ 'log_router' ],

  /**
   * Настройка подключаемых компонентов. Здесь указываются как компоненты autodafe, так и пользовательские. Ключами
   * всегда является название подключаемого компонентка ( для пользовательских компонентов это название файла ), а
   * значениями - настройки для компонента. Если для компонента не надо передавать настройки, нужно просто указать true
   *
   * @type {Object}
   */
  components : {

    // http сервер
    http                : {
      port : 3000
    },

    // настройки логгера
    log_router          : {
      routes : {
        console : {
          levels : [ 'trace', 'info', 'warning', 'error' ]
    } } }
  }
};